@@ -42,7 +42,7 @@ static int file_read(struct burn_source *source,
 		     unsigned char *buffer,
 		     int size)
 {
-	struct burn_source_fd *fs = source->data;
+	struct burn_source_file *fs = source->data;
 
 	return read_full_buffer(fs->datafd, buffer, size);
 }
@@ -71,14 +71,24 @@ static off_t file_size(struct burn_source *source)
 	struct stat buf;
 	struct burn_source_file *fs = source->data;
 
+	if (fs->fixed_size > 0)
+		return fs->fixed_size;
 	if (fstat(fs->datafd, &buf) == -1)
 		return (off_t) 0;
-	/* for now we keep it compatible to the old (int) return value */
-	if(buf.st_size >= 1308622848)  /* 2 GB - 800 MB to prevent rollover */ 
-		return (off_t) 1308622848;
 	return (off_t) buf.st_size;
 }
 
+
+/* ts A70125 */
+static int file_set_size(struct burn_source *source, off_t size)
+{
+	struct burn_source_file *fs = source->data;
+
+	fs->fixed_size = size;
+	return 1;
+}
+
+
 struct burn_source *burn_file_source_new(const char *path, const char *subpath)
 {
 	struct burn_source_file *fs;
@@ -103,84 +113,43 @@ struct burn_source *burn_file_source_new(const char *path, const char *subpath)
 	if (subpath)
 		fs->subfd = fd2;
 
+	/* ts A70125 */
+	fs->fixed_size = 0;
+
 	src = burn_source_new();
 	src->read = file_read;
 	if (subpath)
 		src->read_sub = file_read_sub;
 
 	src->get_size = file_size;
+	src->set_size = file_set_size;
 	src->free_data = file_free;
 	src->data = fs;
 	return src;
 }
 
 
-/* ------ provisory location for the new source subclass fd --------- */
-
-static off_t fd_get_size(struct burn_source *source)
-{
-	struct stat buf;
-	struct burn_source_fd *fs = source->data;
-
-	if (fs->fixed_size > 0)
-		return fs->fixed_size;
-	if (fstat(fs->datafd, &buf) == -1)
-		return (off_t) 0;
-	/* for now we keep it compatible to the old (int) return value */
-	if (buf.st_size >= 1308622848) /* 2 GB - 800 MB to prevent rollover */
-		return (off_t) 1308622848;
-	return buf.st_size;
-}
-
-static int fd_read(struct burn_source *source,
-		     unsigned char *buffer,
-		     int size)
-{
-	struct burn_source_fd *fs = source->data;
-
-	return read_full_buffer(fs->datafd, buffer, size);
-}
-
-
-static int fd_read_sub(struct burn_source *source,
-		     unsigned char *buffer,
-		     int size)
-{
-	struct burn_source_fd *fs = source->data;
-
-	return read_full_buffer(fs->subfd, buffer, size);
-}
-
-
-static void fd_free_data(struct burn_source *source)
-{
-	struct burn_source_fd *fs = source->data;
-
-	close(fs->datafd);
-	if (source->read_sub)
-		close(fs->subfd);
-	free(fs);
-}
-
+/* ts A70126 : removed class burn_source_fd in favor of burn_source_file */
 
 struct burn_source *burn_fd_source_new(int datafd, int subfd, off_t size)
 {
-	struct burn_source_fd *fs;
+	struct burn_source_file *fs;
 	struct burn_source *src;
 
 	if (datafd == -1)
 		return NULL;
-	fs = malloc(sizeof(struct burn_source_fd));
+	fs = malloc(sizeof(struct burn_source_file));
 	fs->datafd = datafd;
 	fs->subfd = subfd;
 	fs->fixed_size = size;
 
 	src = burn_source_new();
-	src->read = fd_read;
+	src->read = file_read;
 	if(subfd != -1)
-		src->read = fd_read_sub;
-	src->get_size = fd_get_size;
-	src->free_data = fd_free_data;
+		src->read = file_read_sub;
+	src->get_size = file_size;
+	src->set_size = file_set_size;
+	src->free_data = file_free;
 	src->data = fs;
 	return src;
 }
