@@ -7,6 +7,11 @@ struct burn_source_file
 {
 	int datafd;
 	int subfd;
+	off_t fixed_size;
 };
 
+
+/* ts A70126 : burn_source_file obsoleted burn_source_fd */
+
+
 #endif /* LIBBURN__FILE_H */
