@@ -1,7 +1,10 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
 #include <stdio.h>
-#include <assert.h>
+
+/* ts A61010 */
+/* #include <a ssert.h> */
+
 #include <unistd.h>
 #include <string.h>
 #include "error.h"
@@ -16,6 +19,15 @@
 #include "toc.h"
 #include "write.h"
 
+
+#ifdef Libburn_log_in_and_out_streaM
+/* <<< ts A61031 */
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#endif /* Libburn_log_in_and_out_streaM */
+
+
 /*static unsigned char isrc[] = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";*/
 
 #define sector_common(X) d->alba++; d->rlba X;
@@ -37,13 +49,15 @@ static void uncook_subs(unsigned char *dest, unsigned char *source)
 	}
 }
 
-/* 0 means "same as inmode" */
-static int get_outmode(struct burn_write_opts *o)
+/* @return >=0 : valid , <0 invalid */
+int sector_get_outmode(enum burn_write_types write_type,
+		    enum burn_block_types block_type)
 {
-	if (o->write_type == BURN_WRITE_SAO)
+	/* ts A61103 : extended SAO condition to TAO */
+	if (write_type == BURN_WRITE_SAO || write_type == BURN_WRITE_TAO)
 		return 0;
 	else
-		switch (o->block_type) {
+		switch (block_type) {
 		case BURN_BLOCK_RAW0:
 			return BURN_MODE_RAW;
 		case BURN_BLOCK_RAW16:
@@ -54,13 +68,37 @@ static int get_outmode(struct burn_write_opts *o)
 			return BURN_MODE_RAW | BURN_SUBCODE_R96;
 		case BURN_BLOCK_MODE1:
 			return BURN_MODE1;
+		default:
+			return -1;
 		}
-	assert(0);		/* return BURN_MODE_UNIMPLEMENTED :) */
+
+	/* ts A61007 : now handled in burn_write_opts_set_write_type() */
+	/* a ssert(0); */	/* return BURN_MODE_UNIMPLEMENTED :) */
 }
 
+/* 0 means "same as inmode" */
+static int get_outmode(struct burn_write_opts *o)
+{
+	/* ts A61007 */
+	return sector_get_outmode(o->write_type, o->block_type);
+
+	/* -1 is prevented by check in burn_write_opts_set_write_type() */
+	/* a ssert(0); */		/* return BURN_MODE_UNIMPLEMENTED :) */
+}
+
+
 static void get_bytes(struct burn_track *track, int count, unsigned char *data)
 {
-	int valid, shortage, curr;
+	int valid, shortage, curr, i, tr;
+
+#ifdef Libburn_log_in_and_out_streaM 
+        /* <<< ts A61031 */
+        static int tee_fd= -1;
+        if(tee_fd==-1)
+                tee_fd= open("/tmp/libburn_sg_readin",
+                                O_WRONLY|O_CREAT|O_TRUNC,S_IRUSR|S_IWUSR);
+#endif /* Libburn_log_in_and_out_streaM */
+
 
 /* no track pointer means we're just generating 0s */
 	if (!track) {
@@ -80,7 +118,7 @@ static void get_bytes(struct burn_track *track, int count, unsigned char *data)
 	shortage = count - valid;
 
 	if (!shortage)
-		return;
+		goto ex;
 
 /* Next we use source data */
 	curr = valid;
@@ -88,16 +126,24 @@ static void get_bytes(struct burn_track *track, int count, unsigned char *data)
 		valid = track->source->read(track->source, data + curr, count - curr);
 	} else valid = 0;
 
-	if (valid == -1) {
+	if (valid <= 0) { /* ts A61031 : extended from (valid == -1) */
 		track->eos = 1;
 		valid = 0;
 	}
+	track->sourcecount += valid;
+
+#ifdef Libburn_log_in_and_out_streaM
+	/* <<< ts A61031 */
+        if(tee_fd!=-1 && valid>0) {
+                write(tee_fd, data + curr, valid);
+        }
+#endif /* Libburn_log_in_and_out_streaM */
 
 	curr += valid;
 	shortage = count - curr;
 
 	if (!shortage)
-		return;
+		goto ex;
 
 /* Before going to the next track, we run through any tail */
 
@@ -113,7 +159,13 @@ static void get_bytes(struct burn_track *track, int count, unsigned char *data)
 	shortage -= valid;
 
 	if (!shortage)
-		return;
+		goto ex;
+
+	/* ts A61031 */
+	if (shortage >= count)
+		track->track_data_done = 1;
+	if (track->open_ended)
+		goto ex;
 
 /* If we're still short, and there's a "next" pointer, we pull from that.
    if that depletes, we'll just fill with 0s.
@@ -128,11 +180,24 @@ static void get_bytes(struct burn_track *track, int count, unsigned char *data)
 			curr += valid;
 		}
 	}
-	if (!shortage)
-		return;
-	memset(data + curr, 0, shortage);
+ex:;
+	/* ts A61024 : general finalizing processing */ 
+	if(shortage)
+		memset(data + curr, 0, shortage); /* this is old icculus.org */
+	if (track->swap_source_bytes == 1) {
+		for (i = 1; i < count; i += 2) {
+			tr = data[i];
+			data[i] = data[i-1];
+			data[i-1] = tr;
+		}
+	}
 }
-static unsigned char *get_sector(struct burn_write_opts *opts, int inmode)
+
+/* ts A61009 : seems to hand out sector start pointer in opts->drive->buffer
+		and to count hand outs as well as reserved bytes */
+/* ts A61101 : added parameter track for counting written bytes */
+static unsigned char *get_sector(struct burn_write_opts *opts, 
+				struct burn_track *track, int inmode)
 {
 	struct burn_drive *d = opts->drive;
 	struct buffer *out = d->buffer;
@@ -144,13 +209,30 @@ static unsigned char *get_sector(struct burn_write_opts *opts, int inmode)
 	if (outmode == 0)
 		outmode = inmode;
 
-	seclen = burn_sector_length(outmode) + burn_subcode_length(outmode);
+	/* ts A61009 : react on eventual failure of burn_sector_length()
+			(should not happen if API tested properly).
+			Ensures out->bytes >= out->sectors  */
+	seclen = burn_sector_length(outmode);
+	if (seclen <= 0)
+		return NULL;
+	seclen += burn_subcode_length(outmode);
 
-	if (out->bytes + (seclen) >= BUFFER_SIZE) {
+	/* ts A61219 : opts->obs is eventually a 32k trigger for DVD */
+	if (out->bytes + seclen > BUFFER_SIZE ||
+	    (opts->obs > 0 && out->bytes + seclen > opts->obs)) {
 		int err;
 		err = d->write(d, d->nwa, out);
 		if (err == BE_CANCELLED)
 			return NULL;
+
+		/* ts A61101 */
+		if(track != NULL) {
+			track->writecount += out->bytes;
+			track->written_sectors += out->sectors;
+		}
+		/* ts A61119 */
+		d->progress.buffered_bytes += out->bytes;
+
 		d->nwa += out->sectors;
 		out->bytes = 0;
 		out->sectors = 0;
@@ -163,8 +245,36 @@ static unsigned char *get_sector(struct burn_write_opts *opts, int inmode)
 	return ret;
 }
 
+/* ts A61031 */
+/* Revoke the counting of the most recent sector handed out by get_sector() */
+static void unget_sector(struct burn_write_opts *opts, int inmode)
+{
+	struct burn_drive *d = opts->drive;
+	struct buffer *out = d->buffer;
+	int outmode;
+	int seclen;
+
+	outmode = get_outmode(opts);
+	if (outmode == 0)
+		outmode = inmode;
+
+	/* ts A61009 : react on eventual failure of burn_sector_length()
+			(should not happen if API tested properly).
+			Ensures out->bytes >= out->sectors  */
+	seclen = burn_sector_length(outmode);
+	if (seclen <= 0)
+		return;
+	seclen += burn_subcode_length(outmode);
+
+	out->bytes -= seclen;
+	out->sectors--;
+}
+
+
 /* either inmode == outmode, or outmode == raw.  anything else is bad news */
-static void convert_data(struct burn_write_opts *o, struct burn_track *track,
+/* ts A61010 : changed type to int in order to propagate said bad news */
+/** @return 1 is ok, <= 0 is failure */
+static int convert_data(struct burn_write_opts *o, struct burn_track *track,
 			 int inmode, unsigned char *data)
 {
 	int outlen, inlen;
@@ -177,14 +287,21 @@ static void convert_data(struct burn_write_opts *o, struct burn_track *track,
 
 	outlen = burn_sector_length(outmode);
 	inlen = burn_sector_length(inmode);
-	assert(outlen >= inlen);
+
+	/* ts A61010 */
+	/* a ssert(outlen >= inlen); */
+	if (outlen < inlen)
+		return 0;
 
 	if ((outmode & BURN_MODE_BITS) == (inmode & BURN_MODE_BITS)) {
 		get_bytes(track, inlen, data);
-		return;
+		return 1;
 	}
 
-	assert(outmode & BURN_MODE_RAW);
+	/* ts A61010 */
+	/* a ssert(outmode & BURN_MODE_RAW); */
+	if (!(outmode & BURN_MODE_RAW))
+		return 0;
 
 	if (inmode & BURN_MODE1)
 		offset = 16;
@@ -192,9 +309,16 @@ static void convert_data(struct burn_write_opts *o, struct burn_track *track,
 		offset = 0;
 	if (inmode & BURN_AUDIO)
 		offset = 0;
-	assert(offset != -1);
+
+	/* ts A61010 */
+	/* a ssert(offset != -1); */
+	if (offset == -1)
+		return 0;
+
 	get_bytes(track, inlen, data + offset);
+	return 1;
 }
+
 static void convert_subs(struct burn_write_opts *o, int inmode,
 			 unsigned char *subs, unsigned char *sector)
 {
@@ -228,6 +352,8 @@ static void convert_subs(struct burn_write_opts *o, int inmode,
 		out[0] = ~out[0];
 		out[1] = ~out[1];
 		break;
+	/* ts A61119 : to silence compiler warnings */
+	default:;
 	}
 }
 
@@ -274,10 +400,12 @@ int sector_toc(struct burn_write_opts *o, int mode)
 	unsigned char *data;
 	unsigned char subs[96];
 
-	data = get_sector(o, mode);
-	if (!data)
+	data = get_sector(o, NULL, mode);
+	if (data == NULL)
+		return 0;
+	/* ts A61010 */
+	if (convert_data(o, NULL, mode, data) <= 0)
 		return 0;
-	convert_data(o, NULL, mode, data);
 	subcode_toc(d, mode, subs);
 	convert_subs(o, mode, subs, data);
 	sector_headers(o, data, mode, 1);
@@ -292,10 +420,12 @@ int sector_pregap(struct burn_write_opts *o,
 	unsigned char *data;
 	unsigned char subs[96];
 
-	data = get_sector(o, mode);
-	if (!data)
+	data = get_sector(o, NULL, mode);
+	if (data == NULL)
+		return 0;
+	/* ts A61010 */
+	if (convert_data(o, NULL, mode, data) <= 0)
 		return 0;
-	convert_data(o, NULL, mode, data);
 	subcode_user(o, subs, tno, control, 0, NULL, 1);
 	convert_subs(o, mode, subs, data);
 	sector_headers(o, data, mode, 0);
@@ -310,10 +440,12 @@ int sector_postgap(struct burn_write_opts *o,
 	unsigned char subs[96];
 	unsigned char *data;
 
-	data = get_sector(o, mode);
-	if (!data)
+	data = get_sector(o, NULL, mode);
+	if (data == NULL)
 		return 0;
-	convert_data(o, NULL, mode, data);
+	/* ts A61010 */
+	if (convert_data(o, NULL, mode, data) <= 0)
+		return 0;;
 /* use last index in track */
 	subcode_user(o, subs, tno, control, 1, NULL, 1);
 	convert_subs(o, mode, subs, data);
@@ -362,7 +494,9 @@ static char char_to_isrc(char c)
 		return 0x11 + (c - 'A');
 	if (c >= 'a' && c <= 'z')
 		return 0x11 + (c - 'a');
-	assert(0);
+
+	/* ts A61008 : obsoleted by test in burn_track_set_isrc() */
+	/* a ssert(0); */
 	return 0;
 }
 
@@ -398,7 +532,8 @@ void subcode_user(struct burn_write_opts *o, unsigned char *subcodes,
 		}
 	}
 
-	assert(qmode == 1 || qmode == 2 || qmode == 3);
+	/* ts A61010 : this cannot happen. Assert for fun ? */
+	/* a ssert(qmode == 1 || qmode == 2 || qmode == 3); */
 
 	switch (qmode) {
 	case 1:
@@ -477,10 +612,12 @@ int sector_lout(struct burn_write_opts *o, unsigned char control, int mode)
 	unsigned char subs[96];
 	unsigned char *data;
 
-	data = get_sector(o, mode);
+	data = get_sector(o, NULL, mode);
 	if (!data)
 		return 0;
-	convert_data(o, NULL, mode, data);
+	/* ts A61010 */
+	if (convert_data(o, NULL, mode, data) <= 0)
+		return 0;
 	subcode_lout(o, control, subs);
 	convert_subs(o, mode, subs, data);
 	sector_headers(o, data, mode, 0);
@@ -494,12 +631,23 @@ int sector_data(struct burn_write_opts *o, struct burn_track *t, int psub)
 	unsigned char subs[96];
 	unsigned char *data;
 
-	data = get_sector(o, t->mode);
+	data = get_sector(o, t, t->mode);
 	if (!data)
 		return 0;
-	convert_data(o, t, t->mode, data);
+	/* ts A61010 */
+	if (convert_data(o, t, t->mode, data) <= 0)
+		return 0;
 
-	if (!t->source->read_sub)
+	/* ts A61031 */
+	if (t->open_ended && t->track_data_done) {
+		unget_sector(o, t->mode);
+		return 2;
+	}
+
+	/* ts A61219 : allow track without .entry */
+	if (t->entry == NULL)
+		;
+	else if (!t->source->read_sub)
 		subcode_user(o, subs, t->entry->point,
 			     t->entry->control, 1, &t->isrc, psub);
 	else if (!t->source->read_sub(t->source, subs, 96))
@@ -542,6 +690,22 @@ int dec_to_bcd(int d)
 	return (top << 4) + bottom;
 }
 
+int sector_headers_is_ok(struct burn_write_opts *o, int mode)
+{
+	if (mode & BURN_AUDIO)	/* no headers for "audio" */
+		return 1;
+	if (o->write_type == BURN_WRITE_SAO)
+		return 1;
+
+	/* ts A61031 */
+	if (o->write_type == BURN_WRITE_TAO)
+		return 1;
+
+	if (mode & BURN_MODE1)
+		return 2;
+	return 0;
+}
+
 void sector_headers(struct burn_write_opts *o, unsigned char *out,
 		    int mode, int leadin)
 {
@@ -550,14 +714,33 @@ void sector_headers(struct burn_write_opts *o, unsigned char *out,
 	int min, sec, frame;
 	int modebyte = -1;
 
+	/* ts A61009 */
+#if 1
+	int ret;
+
+	ret = sector_headers_is_ok(o, mode);
+	if (ret != 2)
+		return;
+	modebyte = 1;
+
+#else
+
 	if (mode & BURN_AUDIO)	/* no headers for "audio" */
 		return;
 	if (o->write_type == BURN_WRITE_SAO)
 		return;
+
+	/* ts A61031 */
+	if (o->write_type == BURN_WRITE_TAO)
+		return;
+
 	if (mode & BURN_MODE1)
 		modebyte = 1;
 
-	assert(modebyte == 1);
+#endif
+
+	/* ts A61009 : now ensured by burn_disc_write_is_ok() */
+	/* a ssert(modebyte == 1); */
 
 	out[0] = 0;
 	memset(out + 1, 0xFF, 10);	/* sync */
@@ -629,13 +812,22 @@ void process_q(struct burn_drive *d, unsigned char *q)
 */
 		break;
 	default:
-		assert(0);
+
+		/* ts A61009 : if reactivated then witout Assert */
+		a ssert(0);
 	}
 }
 #endif
 
 /* this needs more info.  subs in the data? control/adr? */
+
+/* ts A61119 : One should not use inofficial compiler extensions.
+   >>> Some day this function needs to be implemented. At least for now
+       the result does not match the "mode" of cdrecord -toc.
+ */
+/*
 #warning sector_identify needs to be written
+*/
 int sector_identify(unsigned char *data)
 {
 	scramble(data);
