@@ -1,21 +1,25 @@
 
-/*  test/libburner.c , API illustration of burning a single data track to CD */
-/*  Copyright (C) 2005 - 2006 Thomas Schmitt <scdbackup@gmx.net> */
-/*  Provided under GPL, see also "License and copyright aspects" at file end */
+/* test/libburner.c , API illustration of burning data or audio tracks to CD */
+/* Copyright (C) 2005 - 2006 Thomas Schmitt <scdbackup@gmx.net> */
+/* Provided under GPL, see also "License and copyright aspects" at file end */
 
 
 /**                               Overview 
   
   libburner is a minimal demo application for the library libburn as provided
-  on  http://libburn.pykix.org . It can list the available devices, can
-  blank a CD-RW and can burn to CD-R or CD-RW.
+  on  http://libburnia.pykix.org . It can list the available devices, can
+  blank a CD-RW or DVD-RW, can format a DVD-RW, can burn to CD-R, CD-RW, DVD-R,
+  DVD+R, DVD+RW, DVD-RAM or DVD-RW. Not supported yet: double layer media.
+
   It's main purpose, nevertheless, is to show you how to use libburn and also
-  to serve the libburn team as reference application. libburner.c does indeed
+  to serve the libburnia team as reference application. libburner.c does indeed
   define the standard way how above three gestures can be implemented and
   stay upward compatible for a good while.
   
   Before you can do anything, you have to initialize libburn by
      burn_initialize()
+  and provide some signal and abort handling, e.g. by the builtin handler, by
+     burn_set_signal_handling() 
   as it is done in main() at the end of this file. Then you aquire a
   drive in an appropriate way conforming to the API. The two main
   approaches are shown here in application functions:
@@ -23,9 +27,9 @@
      libburner_aquire_by_driveno()      demonstrates a scan-and-choose approach
   With that aquired drive you can blank a CD-RW
      libburner_blank_disc()
-  Between blanking and burning one eventually has to reload the drive status
-     libburner_regrab()
-  With the aquired drive you can burn to CD-R or blank CD-RW
+  or you can format a DVD-RW to profile "Restricted Overwrite" (needed once)
+     libburner_format_row()
+  With the aquired drive you can burn to CD-R, CD-RW, DVD+RW, DVD-RAM, DVD-RW
      libburner_payload()
   When everything is done, main() releases the drive and shuts down libburn:
      burn_drive_release();
@@ -33,17 +37,10 @@
   
 */
 
-/* We shall prepare for times when more than 2 GB of data are to be handled.
-   This gives POSIX-ly 64 bit off_t */
-#ifndef _LARGEFILE_SOURCE
-#define _LARGEFILE_SOURCE 1
-#endif
-#ifndef _FILE_OFFSET_BITS
-#define _FILE_OFFSET_BITS 64
-#endif
-
 /** See this for the decisive API specs . libburn.h is The Original */
-#include <libburn/libburn.h>
+/*  For using the installed header file :  #include <libburn/libburn.h> */
+/*  This program insists in the own headerfile. */
+#include "../libburn/libburn.h"
 
 /* libburn is intended for Linux systems with kernel 2.4 or 2.6 for now */
 #include <stdio.h>
@@ -54,6 +51,8 @@
 #include <stdlib.h>
 #include <time.h>
 #include <errno.h>
+#include <sys/stat.h>
+#include <fcntl.h>
 
 
 /** For simplicity i use global variables to represent the drives.
@@ -73,12 +72,9 @@ static unsigned int drive_count;
     finally released */
 static int drive_is_grabbed = 0;
 
-
-/** Here you may enable simulated burn by default. This does not apply to
-    blanking. Anyway, some CD recorders obey the request to simulate, some do
-    not. Explicit options are:  --burn_for_real  and  --try_to_simulate
-*/
-static int simulate_burn = 0;
+/** A number and a text describing the type of media in aquired drive */
+static int current_profile= -1;
+static char current_profile_name[80]= {""};
 
 
 /* Some in-advance definitions to allow a more comprehensive ordering
@@ -108,7 +104,13 @@ int libburner_aquire_drive(char *drive_adr, int *driveno)
 		ret = libburner_aquire_by_adr(drive_adr);
 	else
 		ret = libburner_aquire_by_driveno(driveno);
-	return ret;
+	if (ret <= 0)
+		return ret;
+	burn_disc_get_profile(drive_list[0].drive, &current_profile,
+				 current_profile_name);
+	if (current_profile_name[0])
+		printf("Detected media type: %s\n", current_profile_name);
+	return 1;
 }
 
 
@@ -119,15 +121,21 @@ int libburner_aquire_drive(char *drive_adr, int *driveno)
 int libburner_aquire_by_adr(char *drive_adr)
 {
 	int ret;
-	
-	printf("Aquiring drive '%s' ...\n",drive_adr);
-	ret = burn_drive_scan_and_grab(&drive_list,drive_adr,1);
+	char libburn_drive_adr[BURN_DRIVE_ADR_LEN];
+
+	/* This tries to resolve links or alternative device files */
+	ret = burn_drive_convert_fs_adr(drive_adr, libburn_drive_adr);	
+	if (ret<=0) {
+		fprintf(stderr,"Address does not lead to a CD burner: '%s'\n",
+			drive_adr);
+		return ret;
+	}
+
+	printf("Aquiring drive '%s' ...\n",libburn_drive_adr);
+	ret = burn_drive_scan_and_grab(&drive_list,libburn_drive_adr,1);
 	if (ret <= 0) {
 		fprintf(stderr,"FAILURE with persistent drive address  '%s'\n",
-			drive_adr);
-		if (strncmp(drive_adr,"/dev/sg",7) != 0 &&
-		    strncmp(drive_adr,"/dev/hd",7) != 0)
-			fprintf(stderr,"\nHINT: Consider addresses like  '/dev/hdc'  or  '/dev/sg0'\n");
+			libburn_drive_adr);
 	} else {
 		printf("Done\n");
 		drive_is_grabbed = 1;
@@ -187,7 +195,6 @@ int libburner_aquire_by_driveno(int *driveno)
 	}
 	printf("-----------------------------------------------------------------------------\n\n");
 
-
 	/*
 	On multi-drive systems save yourself from sysadmins' revenge.
 
@@ -248,37 +255,34 @@ int libburner_aquire_by_driveno(int *driveno)
 
     To our knowledge it is hardly possible to abort an ongoing blank operation
     because after start it is entirely handled by the drive.
-    So expect a blank run to survive the end of the blanking process and be
-    patient for the usual timespan of a normal blank run. Only after that
-    time has surely elapsed, only then you should start any rescue attempts
-    with the drive. If necessary at all.
+    So expect signal handling to wait the normal blanking timespan until it
+    can allow the process to end. External kill -9 will not help the drive.
 */
 int libburner_blank_disc(struct burn_drive *drive, int blank_fast)
 {
 	enum burn_disc_status disc_state;
-	struct burn_progress progress;
+	struct burn_progress p;
+	double percent = 1.0;
 
-	while (burn_drive_get_status(drive, NULL))
-		usleep(1001);
-
-	while ((disc_state = burn_disc_get_status(drive)) == BURN_DISC_UNREADY)
-		usleep(1001);
+	disc_state = burn_disc_get_status(drive);
 	printf(
 	    "Drive media status:  %d  (see  libburn/libburn.h  BURN_DISC_*)\n",
 	    disc_state);
-	if (disc_state == BURN_DISC_BLANK) {
+	if (current_profile == 0x13) {
+		; /* formatted DVD-RW will get blanked to sequential state */
+	} else if (disc_state == BURN_DISC_BLANK) {
 		fprintf(stderr,
-		  "IDLE: Blank CD media detected. Will leave it untouched\n");
+		  "IDLE: Blank media detected. Will leave it untouched\n");
 		return 2;
 	} else if (disc_state == BURN_DISC_FULL ||
 		   disc_state == BURN_DISC_APPENDABLE) {
-		; /* this is what libburn is willing to blank */
+		; /* this is what libburner is willing to blank */
 	} else if (disc_state == BURN_DISC_EMPTY) {
 		fprintf(stderr,"FATAL: No media detected in drive\n");
 		return 0;
 	} else {
 		fprintf(stderr,
-			"FATAL: Cannot recognize drive and media state\n");
+			"FATAL: Unsuitable drive and media state\n");
 		return 0;
 	}
 	if(!burn_disc_erasable(drive)) {
@@ -287,12 +291,14 @@ int libburner_blank_disc(struct burn_drive *drive, int blank_fast)
 		return 0;
 	}
 	printf(
-	      "Beginning to %s-blank CD media.\n", (blank_fast?"fast":"full"));
-	printf(
-	      "Expect some garbage sector numbers and some zeros at first.\n");
+	      "Beginning to %s-blank media.\n", (blank_fast?"fast":"full"));
 	burn_disc_erase(drive, blank_fast);
-	while (burn_drive_get_status(drive, &progress)) {
-		printf("Blanking sector  %d\n", progress.sector);
+	sleep(1);
+	while (burn_drive_get_status(drive, &p) != BURN_DRIVE_IDLE) {
+		if(p.sectors>0 && p.sector>=0) /* display 1 to 99 percent */
+			percent = 1.0 + ((double) p.sector+1.0)
+					 / ((double) p.sectors) * 98.0;
+		printf("Blanking  ( %.1f%% done )\n", percent);
 		sleep(1);
 	}
 	printf("Done\n");
@@ -300,136 +306,154 @@ int libburner_blank_disc(struct burn_drive *drive, int blank_fast)
 }
 
 
-/** This gesture is necessary to get the drive info after blanking.
-    It opens a small gap for losing the drive to another libburn instance.
-    We will work on closing this gap.
-*/
-int libburner_regrab(struct burn_drive *drive) {
-	int ret;
+/** Persistently changes DVD-RW profile 0014h "Sequential Recording" to
+    profile 0013h "Restricted Overwrite" which needs no blanking for re-use
+    but is not capable of multi-session.
 
-	printf("Releasing and regrabbing drive ...\n");
-	if (drive_is_grabbed)
-		burn_drive_release(drive, 0);
-	drive_is_grabbed = 0;
-	ret = burn_drive_grab(drive, 0);
-	if (ret != 0) {
-		drive_is_grabbed = 1;
-	 	printf("Done\n");
-	} else
-	 	printf("FAILED\n");
-	return !!ret;
+    Expect a behavior similar to blanking with unusual noises from the drive.
+*/
+int libburner_format_row(struct burn_drive *drive)
+{
+	struct burn_progress p;
+	double percent = 1.0;
+
+	if (current_profile == 0x13) {
+		fprintf(stderr, "IDLE: DVD-RW media is already formatted\n");
+		return 2;
+	} else if (current_profile != 0x14) {
+		fprintf(stderr, "FATAL: Can only format DVD-RW\n");
+		return 0;
+	}
+	printf("Beginning to format media.\n");
+	burn_disc_format(drive, (off_t) 0, 0);
+
+	sleep(1);
+	while (burn_drive_get_status(drive, &p) != BURN_DRIVE_IDLE) {
+		if(p.sectors>0 && p.sector>=0) /* display 1 to 99 percent */
+			percent = 1.0 + ((double) p.sector+1.0)
+					 / ((double) p.sectors) * 98.0;
+		printf("Formatting  ( %.1f%% done )\n", percent);
+		sleep(1);
+	}
+	burn_disc_get_profile(drive_list[0].drive, &current_profile,
+				 current_profile_name);
+	printf("Media type now: %4.4xh  \"%s\"\n",
+		 current_profile, current_profile_name);
+	if (current_profile != 0x13) {
+		fprintf(stderr,
+		  "FATAL: Failed to change media profile to desired value\n");
+		return 0;
+	}
+	return 1;
 }
 
 
-/** Brings the preformatted image (ISO 9660, afio, ext2, whatever) onto media.
+/** Brings preformatted track images (ISO 9660, audio, ...) onto media.
+    To make sure a data image is fully readable on any Linux machine, this
+    function adds 300 kiB of padding to the (usualy single) track.
+    Audio tracks get padded to complete their last sector.
 
-    To make sure your image is fully readable on any Linux machine, this
-    function adds 300 kB of padding to the track.
+    In case of external signals expect abort handling of an ongoing burn to
+    last up to a minute. Wait the normal burning timespan before any kill -9.
 
-    Without a signal handler it is quite dangerous to abort the process
-    while this function is active. See cdrskin/cdrskin.c and its usage
-    of cdrskin/cleanup.[ch] for an example of application provided
-    abort handling. It must cope with 2 of 3 threads reporting for
-    being handled.
-
-    Without signal handler have ready a command line 
-       cdrecord dev=... -reset
-    with a dev= previously inquired by cdrecord [dev=ATA] -scanbus 
-    in order to get your drive out of shock state after raw abort.
-    Thanks to Joerg Schilling for helping out unquestioned. :) 
-
-    In general, libburn is less prone to system problems than cdrecord,
-    i believe. But cdrecord had long years of time to complete itself.
-    We are still practicing. Help us with that. :))
+    For simplicity, this function allows memory leaks in case of failure.
+    In apps which do not abort immediately, one should clean up better.
 */
-int libburner_payload(struct burn_drive *drive, const char *source_adr,
-		     off_t size)
+int libburner_payload(struct burn_drive *drive, 
+		      char source_adr[][4096], int source_adr_count,
+		      int multi, int simulate_burn, int all_tracks_type)
 {
 	struct burn_source *data_src;
 	struct burn_disc *target_disc;
 	struct burn_session *session;
 	struct burn_write_opts *burn_options;
 	enum burn_disc_status disc_state;
-	struct burn_track *track;
+	struct burn_track *track, *tracklist[99];
 	struct burn_progress progress;
 	time_t start_time;
-	int last_sector = 0;
+	int last_sector = 0, padding = 0, trackno, unpredicted_size = 0, fd;
+	off_t fixed_size;
+	char *adr, reasons[BURN_REASONS_LEN];
+	struct stat stbuf;
+
+	if (all_tracks_type != BURN_AUDIO) {
+		all_tracks_type = BURN_MODE1;
+		/* a padding of 300 kiB helps to avoid the read-ahead bug */
+		padding = 300*1024;
+	}
 
 	target_disc = burn_disc_create();
 	session = burn_session_create();
 	burn_disc_add_session(target_disc, session, BURN_POS_END);
-	track = burn_track_create();
 
-	/* a padding of 300 kB is helpful to avoid the read-ahead bug */
-	burn_track_define_data(track, 0, 300*1024, 1, BURN_MODE1);
+	for (trackno = 0 ; trackno < source_adr_count; trackno++) {
+	  tracklist[trackno] = track = burn_track_create();
+	  burn_track_define_data(track, 0, padding, 1, all_tracks_type);
 
-	if (source_adr[0] == '-' && source_adr[1] == 0) {
-		data_src = burn_fd_source_new(0, -1, size);
-		printf("Note: using standard input as source with %.f bytes\n",
-			(double) size);
-	} else
-		data_src = burn_file_source_new(source_adr, NULL);
-	if (data_src == NULL) {
+	  adr = source_adr[trackno];
+	  fixed_size = 0;
+	  if (adr[0] == '-' && adr[1] == 0) {
+		fd = 0;
+	  } else {
+		fd = open(adr, O_RDONLY);
+		if (fd>=0)
+			if (fstat(fd,&stbuf)!=-1)
+				if((stbuf.st_mode&S_IFMT)==S_IFREG)
+					fixed_size = stbuf.st_size;
+	  }
+	  if (fixed_size==0)
+		unpredicted_size = 1;
+	  data_src = NULL;
+	  if (fd>=0)
+	  	data_src = burn_fd_source_new(fd, -1, fixed_size);
+	  if (data_src == NULL) {
 		fprintf(stderr,
-		       "FATAL: Could not open data source '%s'.\n",source_adr);
+		       "FATAL: Could not open data source '%s'.\n",adr);
 		if(errno!=0)
 			fprintf(stderr,"(Most recent system error: %s )\n",
 				strerror(errno));
 		return 0;
-	}
-
-	if (burn_track_set_source(track, data_src) != BURN_SOURCE_OK) {
+	  }
+	  if (burn_track_set_source(track, data_src) != BURN_SOURCE_OK) {
 		printf("FATAL: Cannot attach source object to track object\n");
 		return 0;
-	}
-	burn_session_add_track(session, track, BURN_POS_END);
-	burn_source_free(data_src);
+	  }
 
-	while (burn_drive_get_status(drive, NULL))
-		usleep(1001);
+	  burn_session_add_track(session, track, BURN_POS_END);
+	  printf("Track %d : source is '%s'\n", trackno+1, adr);
+	  burn_source_free(data_src);
+        } /* trackno loop end */
 
 	/* Evaluate drive and media */
-	while ((disc_state = burn_disc_get_status(drive)) == BURN_DISC_UNREADY)
-		usleep(1001);
-	if (disc_state != BURN_DISC_BLANK) {
-		if (disc_state == BURN_DISC_FULL ||
-		    disc_state == BURN_DISC_APPENDABLE) {
-			fprintf(stderr,
-      		       "FATAL: Media with data detected. Need blank media.\n");
+	disc_state = burn_disc_get_status(drive);
+	if (disc_state != BURN_DISC_BLANK &&
+	    disc_state != BURN_DISC_APPENDABLE) {
+		if (disc_state == BURN_DISC_FULL) {
+			fprintf(stderr, "FATAL: Closed media with data detected. Need blank or appendable media.\n");
 			if (burn_disc_erasable(drive))
 				fprintf(stderr, "HINT: Try --blank_fast\n\n");
 		} else if (disc_state == BURN_DISC_EMPTY) 
 			fprintf(stderr,"FATAL: No media detected in drive\n");
 		else
 			fprintf(stderr,
-			    "FATAL: Cannot recognize drive and media state\n");
+			 "FATAL: Cannot recognize state of drive and media\n");
 		return 0;
 	}
 
 	burn_options = burn_write_opts_new(drive);
 	burn_write_opts_set_perform_opc(burn_options, 0);
-
-#ifdef Libburner_raw_mode_which_i_do_not_likE
-	/* This yields higher CD capacity but hampers my IDE controller
-	   with burning on one drive and reading on another simultaneously.
-	   My burner does not obey the order --try_to_simulate in this mode.
-        */
-	burn_write_opts_set_write_type(burn_options,
-				       BURN_WRITE_RAW, BURN_BLOCK_RAW96R);
-#else
-
-	/* This is by what cdrskin competes with cdrecord -sao which
-	   i understand is the mode preferrably advised by Joerg Schilling */
-	burn_write_opts_set_write_type(burn_options,
-				       BURN_WRITE_SAO, BURN_BLOCK_SAO);
-
-#endif
+	burn_write_opts_set_multi(burn_options, !!multi);
 	if(simulate_burn)
 		printf("\n*** Will TRY to SIMULATE burning ***\n\n");
 	burn_write_opts_set_simulate(burn_options, simulate_burn);
-	burn_structure_print_disc(target_disc);
 	burn_drive_set_speed(drive, 0, 0);
 	burn_write_opts_set_underrun_proof(burn_options, 1);
+	if (burn_write_opts_auto_write_type(burn_options, target_disc,
+					reasons, 0) == BURN_WRITE_NONE) {
+		fprintf(stderr, "FATAL: Failed to find a suitable write mode with this media.\n");
+		fprintf(stderr, "Reasons given:\n%s\n", reasons);
+		return 0;
+	}
 
 	printf("Burning starts. With e.g. 4x media expect up to a minute of zero progress.\n");
 	start_time = time(0);
@@ -438,49 +462,61 @@ int libburner_payload(struct burn_drive *drive, const char *source_adr,
 	burn_write_opts_free(burn_options);
 	while (burn_drive_get_status(drive, NULL) == BURN_DRIVE_SPAWNING)
 		usleep(1002);
-	while (burn_drive_get_status(drive, &progress)) {
+	while (burn_drive_get_status(drive, &progress) != BURN_DRIVE_IDLE) {
 		if( progress.sectors <= 0 || progress.sector == last_sector)
 			printf(
 			     "Thank you for being patient since %d seconds.\n",
 			     (int) (time(0) - start_time));
+		else if(unpredicted_size)
+			printf("Track %d : sector %d\n", progress.track+1,
+				progress.sector);
 		else
-			printf("Burning sector %d of %d\n",
+			printf("Track %d : sector %d of %d\n",progress.track+1,
 				progress.sector, progress.sectors);
 		last_sector = progress.sector;
 		sleep(1);
 	}
 	printf("\n");
-	burn_track_free(track);
+
+	for (trackno = 0 ; trackno < source_adr_count; trackno++)
+		burn_track_free(tracklist[trackno]);
 	burn_session_free(session);
 	burn_disc_free(target_disc);
-	if(simulate_burn)
+	if (multi && current_profile != 0x1a && current_profile != 0x13 &&
+		current_profile != 0x12) /* not with DVD+RW, DVD-RW, DVD-RAM */
+		printf("NOTE: Media left appendable.\n");
+	if (simulate_burn)
 		printf("\n*** Did TRY to SIMULATE burning ***\n\n");
-	return 0;
+	return 1;
 }
 
 
-/** Converts command line arguments into a few program parameters.
-    drive_adr[] must provide at least BURN_DRIVE_ADR_LEN bytes.
-    source_adr[] must provide at least 4096 bytes.
-*/
-int libburner_setup(int argc, char **argv, char drive_adr[], int *driveno,
-                   int *do_blank, char source_adr[], off_t *size)
-{
-    int i, insuffient_parameters = 0;
-    int print_help = 0;
+/** The setup parameters of libburner */
+static char drive_adr[BURN_DRIVE_ADR_LEN] = {""};
+static int driveno = 0;
+static int do_blank = 0;
+static char source_adr[99][4096];
+static int source_adr_count = 0;
+static int do_multi = 0;
+static int simulate_burn = 0;
+static int all_tracks_type = BURN_MODE1;
 
-    drive_adr[0] = 0;
-    *driveno = 0;
-    *do_blank = 0;
-    source_adr[0] = 0;
-    *size = 650*1024*1024;
+
+/** Converts command line arguments into above setup parameters.
+*/
+int libburner_setup(int argc, char **argv)
+{
+    int i, insuffient_parameters = 0, print_help = 0;
 
     for (i = 1; i < argc; ++i) {
-        if (!strcmp(argv[i], "--blank_fast")) {
-            *do_blank = 1;
+        if (!strcmp(argv[i], "--audio")) {
+            all_tracks_type = BURN_AUDIO;
+
+        } else if (!strcmp(argv[i], "--blank_fast")) {
+            do_blank = 1;
 
         } else if (!strcmp(argv[i], "--blank_full")) {
-            *do_blank = 2;
+            do_blank = 2;
 
         } else if (!strcmp(argv[i], "--burn_for_real")) {
             simulate_burn = 0;
@@ -492,10 +528,10 @@ int libburner_setup(int argc, char **argv, char drive_adr[], int *driveno,
                 return 1;
             } else if (strcmp(argv[i], "-") == 0) {
                 drive_adr[0] = 0;
-                *driveno = -1;
+                driveno = -1;
             } else if (isdigit(argv[i][0])) {
                 drive_adr[0] = 0;
-                *driveno = atoi(argv[i]);
+                driveno = atoi(argv[i]);
             } else {
                 if(strlen(argv[i]) >= BURN_DRIVE_ADR_LEN) {
                     fprintf(stderr,"--drive address too long (max. %d)\n",
@@ -504,65 +540,71 @@ int libburner_setup(int argc, char **argv, char drive_adr[], int *driveno,
                 }
                 strcpy(drive_adr, argv[i]);
             }
-        } else if (!strcmp(argv[i], "--stdin_size")) {
-            ++i;
-            if (i >= argc) {
-                fprintf(stderr,"--stdin_size requires an argument\n");
-                return 3;
-            } else
-                *size = atoi(argv[i]);
-            if (*size < 600*1024) /* seems to be minimum readable track size */
-                *size = 600*1024;
+        } else if (!strcmp(argv[i], "--format_overwrite")) {
+            do_blank = 101;
+
+        } else if (!strcmp(argv[i], "--multi")) {
+	    do_multi = 1;
+
+	} else if (!strcmp(argv[i], "--stdin_size")) { /* obsoleted */
+	    i++;
+
         } else if (!strcmp(argv[i], "--try_to_simulate")) {
             simulate_burn = 1;
 
-        } else if (!strcmp(argv[i], "--verbose")) {
-            ++i;
-            if (i >= argc) {
-                fprintf(stderr,"--verbose requires an argument\n");
-                return 4;
-            } else
-                burn_set_verbosity(atoi(argv[i]));
         } else if (!strcmp(argv[i], "--help")) {
             print_help = 1;
 
+        } else if (!strncmp(argv[i], "--",2)) {
+            fprintf(stderr, "Unidentified option: %s\n", argv[i]);
+            return 7;
         } else {
             if(strlen(argv[i]) >= 4096) {
                 fprintf(stderr, "Source address too long (max. %d)\n", 4096-1);
                 return 5;
             }
-            strcpy(source_adr, argv[i]);
+            if(source_adr_count >= 99) {
+                fprintf(stderr, "Too many tracks (max. 99)\n");
+                return 6;
+            }
+            strcpy(source_adr[source_adr_count], argv[i]);
+            source_adr_count++;
         }
     }
     insuffient_parameters = 1;
-    if (*driveno < 0)
+    if (driveno < 0)
         insuffient_parameters = 0;
-    if (source_adr[0] != 0)
+    if (source_adr_count > 0)
         insuffient_parameters = 0; 
-    if (*do_blank)
+    if (do_blank)
         insuffient_parameters = 0;
     if (print_help || insuffient_parameters ) {
         printf("Usage: %s\n", argv[0]);
-        printf("       [--drive <address>|<driveno>|\"-\"]\n");
-        printf("       [--verbose <level>] [--blank_fast|--blank_full]\n");
-        printf("       [--burn_for_real|--try_to_simulate] [--stdin_size <bytes>]\n");
-        printf("       [<imagefile>|\"-\"]\n");
+        printf("       [--drive <address>|<driveno>|\"-\"]  [--audio]\n");
+        printf("       [--blank_fast|--blank_full|--format_overwrite]\n");
+	printf("       [--try_to_simulate]\n");
+        printf("       [--multi]  [<one or more imagefiles>|\"-\"]\n");
         printf("Examples\n");
         printf("A bus scan (needs rw-permissions to see a drive):\n");
         printf("  %s --drive -\n",argv[0]);
-        printf("Burn a file to drive chosen by number:\n");
-        printf("  %s --drive 0 --burn_for_real my_image_file\n",
-            argv[0]);
-        printf("Burn a file to drive chosen by persistent address:\n");
-        printf("  %s --drive /dev/hdc --burn_for_real my_image_file\n", argv[0]);
+        printf("Burn a file to drive chosen by number, leave appendable:\n");
+        printf("  %s --drive 0 --multi my_image_file\n", argv[0]);
+        printf("Burn a file to drive chosen by persistent address, close:\n");
+        printf("  %s --drive /dev/hdc my_image_file\n", argv[0]);
         printf("Blank a used CD-RW (is combinable with burning in one run):\n");
-        printf("  %s --drive 0 --blank_fast\n",argv[0]);
-        printf("Burn a compressed afio archive on-the-fly, pad up to 700 MB:\n");
+        printf("  %s --drive /dev/hdc --blank_fast\n",argv[0]);
+        printf("Blank a used DVD-RW (is combinable with burning in one run):\n");
+        printf("  %s --drive /dev/hdc --blank_full\n",argv[0]);
+        printf("Format a DVD-RW to avoid need for blanking before re-use:\n");
+        printf("  %s --drive /dev/hdc --format_overwrite\n", argv[0]);
+        printf("Burn two audio tracks (to CD only):\n");
+        printf("  lame --decode -t /path/to/track1.mp3 track1.cd\n");
+        printf("  test/dewav /path/to/track2.wav -o track2.cd\n");
+        printf("  %s --drive /dev/hdc --audio track1.cd track2.cd\n", argv[0]);
+        printf("Burn a compressed afio archive on-the-fly:\n");
         printf("  ( cd my_directory ; find . -print | afio -oZ - ) | \\\n");
-        printf("  %s --drive /dev/hdc --burn_for_real --stdin_size 734003200  -\n", argv[0]);
-        printf("To be read from *not mounted* CD via:\n");
-        printf("   afio -tvZ /dev/hdc\n");
-        printf("Program tar would need a clean EOF which our padded CD cannot deliver.\n");
+        printf("  %s --drive /dev/hdc -\n", argv[0]);
+        printf("To be read from *not mounted* media via: afio -tvZ /dev/hdc\n");
         if (insuffient_parameters)
             return 6;
     }
@@ -572,24 +614,28 @@ int libburner_setup(int argc, char **argv, char drive_adr[], int *driveno,
 
 int main(int argc, char **argv)
 {
-	int driveno, ret, do_blank;
-	char source_adr[4096], drive_adr[BURN_DRIVE_ADR_LEN];
-	off_t stdin_size;
+	int ret;
 
-	ret = libburner_setup(argc, argv, drive_adr, &driveno, &do_blank, 
-		       source_adr, &stdin_size);
+	ret = libburner_setup(argc, argv);
 	if (ret)
 		exit(ret);
 
-	printf("Initializing library ...\n");
+	printf("Initializing libburnia.pykix.org ...\n");
 	if (burn_initialize())
 		printf("Done\n");
 	else {
 		printf("FAILED\n");
-		fprintf(stderr,"\nFATAL: Failed to initialize libburn.\n");
+		fprintf(stderr,"\nFATAL: Failed to initialize.\n");
 		exit(33);
 	}
 
+	/* Print messages of severity SORRY or more directly to stderr */
+	burn_msgs_set_severities("NEVER", "SORRY", "libburner : ");
+
+	/* Activate the default signal handler which eventually will try to
+	   properly shutdown drive and library on aborting events. */
+	burn_set_signal_handling("libburner : ", NULL, 0);
+
 	/** Note: driveno might change its value in this call */
 	ret = libburner_aquire_drive(drive_adr, &driveno);
 	if (ret<=0) {
@@ -599,21 +645,18 @@ int main(int argc, char **argv)
 	if (ret == 2)
 		{ ret = 0; goto release_drive; }
 	if (do_blank) {
-		ret = libburner_blank_disc(drive_list[driveno].drive,
-					  do_blank == 1);
+		if (do_blank > 100)
+			ret = libburner_format_row(drive_list[driveno].drive);
+		else
+			ret = libburner_blank_disc(drive_list[driveno].drive,
+							do_blank == 1);
 		if (ret<=0)
 			{ ret = 36; goto release_drive; }
-		if (ret != 2 && source_adr[0] != 0)
-			ret = libburner_regrab(drive_list[driveno].drive);
-		if (ret<=0) {
-			fprintf(stderr,
-	        "FATAL: Cannot release and grab again drive after blanking\n");
-			{ ret = 37; goto finish_libburn; }
-		}
 	}
-	if (source_adr[0] != 0) {
-		ret = libburner_payload(drive_list[driveno].drive, source_adr,
-				 stdin_size);
+	if (source_adr_count > 0) {
+		ret = libburner_payload(drive_list[driveno].drive,
+				source_adr, source_adr_count,
+				do_multi, simulate_burn, all_tracks_type);
 		if (ret<=0)
 			{ ret = 38; goto release_drive; }
 	}
@@ -628,7 +671,7 @@ finish_libburn:;
 	/* burn_drive_info_free(drive_list); */
 
 	burn_finish();
-	return ret;
+	exit(ret);
 }
 
 
@@ -641,7 +684,7 @@ Be also invited to study the code of cdrskin/cdrskin.c et al.
 
 
 Clarification in my name and in the name of Mario Danic, copyright holder
-on toplevel of libburn. To be fully in effect after the remaining other
+on toplevel of libburnia. To be fully in effect after the remaining other
 copyrighted code has been replaced by ours and by copyright-free contributions
 of our friends:
 
